from datetime import datetime
from typing import TypeVar, Generic, Type, Optional
from uuid import UUID

from unipipeline.errors.uni_payload_error import UniAnswerPayloadParsingError
from unipipeline.message.uni_message import UniMessage
from unipipeline.message_meta.uni_message_meta import UniMessageMeta

TMessage = TypeVar('TMessage', bound=UniMessage)


class UniAnswerMessage(Generic[TMessage]):

    def __init__(self, meta: UniMessageMeta, message_payload_type: Type[TMessage]) -> None:
        self._meta = meta
        self._message_payload_type = message_payload_type
        self._message_payload_cache: Optional[TMessage] = None

    @property
    def id(self) -> UUID:
        return self._meta.id

    @property
    def date_created(self) -> datetime:
        return self._meta.date_created

    @property
    def worker_creator(self) -> Optional[str]:
        return self._meta.worker_creator

    @property
    def payload(self) -> TMessage:
        if isinstance(self._message_payload_cache, UniMessage):
            return self._message_payload_cache  # type: ignore

        try:
            self._message_payload_cache = self._message_payload_type(**self._meta.payload)  # type: ignore
        except Exception as e:
            raise UniAnswerPayloadParsingError(str(e))

        assert self._message_payload_cache is not None

        return self._message_payload_cache



from typing import Callable, Any, Dict

from jinja2 import Environment, BaseLoader, Template


class UniUtilTemplate:

    def __init__(self) -> None:
        self._jinja2_env = Environment(loader=BaseLoader())
        self._templates_index: Dict[int, Template] = dict()

    def set_filter(self, name: str, filter_fn: Callable[..., str]) -> None:
        self._jinja2_env.filters[name] = filter_fn

    def template(self, definition: str, **kwargs: Any) -> str:
        if not isinstance(definition, str):
            raise TypeError(f"definition must be str. {type(definition)} given")
        i = hash(definition)
        if i not in self._templates_index:
            self._templates_index[i] = self._jinja2_env.from_string(definition)
        return self._templates_index[i] .render(**kwargs)



from pydantic import BaseModel

from unipipeline.message_meta.uni_message_meta_error_topic import UniMessageMetaErrTopic


class UniMessageMetaErr(BaseModel):
    error_topic: UniMessageMetaErrTopic
    error_type: str
    error_message: str
    retry_times: int

    class Config:
        frozen = True
        extra = 'forbid'



class UniUtilColor:
    COLOR_MAGENTA = '\u001b[35m'
    COLOR_YELLOW = '\u001b[33m'
    COLOR_GREEN = '\u001b[32m'
    COLOR_WHITE = '\u001b[37m'
    COLOR_BLACK = '\u001b[30m'
    COLOR_GRAY = '\u001b[0;37m'
    COLOR_BLUE = '\u001b[34m'
    COLOR_CYAN = '\u001b[36m'
    COLOR_RED = '\u001b[31m'

    COLOR_RESET = '\u001b[0m'

    def __init__(self, enabled_colors: bool = True) -> None:
        self._enabled = enabled_colors

    @property
    def enabled(self) -> bool:
        return self._enabled

    @enabled.setter
    def enabled(self, enabled: bool) -> None:
        self._enabled = enabled

    def color_it(self, color: str, text: str) -> str:
        return f'{color}{text}{self.COLOR_RESET}' if self._enabled else text



import os.path
import sys

CWD = os.path.dirname(os.path.abspath(__file__))

sys.path.append(os.path.dirname(CWD))

from unipipeline import Uni
from example.args import args

u = Uni(f"{CWD}/dag-{args.type}.yml")

u.scaffold()

u.check()

u.initialize(everything=True)



import argparse


parser = argparse.ArgumentParser()
parser.add_argument('--type', dest='type', choices=('mem', 'kafka', 'rmq'), type=str)
parser.add_argument('--count', dest='produce_count', type=int, required=False, default=3)
parser.add_argument('--worker', dest='worker', type=str, required=False, default=None)

args = parser.parse_args()
